#![warn(clippy::pedantic)]

mod auth;

use std::{fs::File, sync::LazyLock};

use color_eyre::eyre::Result;
use reqwest::header::{HeaderMap, USER_AGENT};
use tracing::level_filters::LevelFilter;
use tracing_subscriber::{
    fmt::{self, time::ChronoLocal},
    layer::SubscriberExt,
    util::SubscriberInitExt,
    EnvFilter, Layer,
};

const VERSION: &str = env!("CARGO_PKG_VERSION");

static HTTP_CLIENT: LazyLock<reqwest::Client> = LazyLock::new(|| {
    let mut headers = HeaderMap::new();
    headers.insert(
        USER_AGENT,
        format!("uku3lig/uklient/{VERSION}").parse().unwrap(),
    );

    reqwest::Client::builder()
        .default_headers(headers)
        .build()
        .unwrap()
});

#[tokio::main]
async fn main() -> Result<()> {
    color_eyre::install()?;
    crate::init_tracing()?;

    tracing::info!("starting...");
    tracing::debug!("starting debug");

    let profile = auth::login().await?;

    tracing::info!("logged in as {} ({})", profile.name, profile.id);

    Ok(())
}

fn init_tracing() -> Result<()> {
    let env_filter = EnvFilter::builder()
        .with_default_directive(LevelFilter::INFO.into())
        .from_env_lossy();

    let log_file = File::create("latest.log")?;

    tracing_subscriber::registry()
        .with(
            fmt::layer()
                .with_timer(ChronoLocal::new("[%H:%M:%S%.3f]".to_string()))
                .with_filter(env_filter),
        )
        .with(
            fmt::layer()
                .with_ansi(false)
                .with_timer(ChronoLocal::new("[%FT%H:%M:%S%.3f]".to_string()))
                .with_writer(log_file)
                .with_filter(LevelFilter::DEBUG),
        )
        .init();

    Ok(())
}
